using System.Collections.Concurrent;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Encompass
{
    internal class MessageManager
    {
        private readonly Dictionary<Type, ConcurrentStack<IMessage>> messageTypeToMessages = new Dictionary<Type, ConcurrentStack<IMessage>>();

        internal void RegisterMessageType(Type messageType)
        {
            if (!messageTypeToMessages.ContainsKey(messageType)) {
                messageTypeToMessages.Add(messageType, new ConcurrentStack<IMessage>());
            }
        }

        internal void AddMessage(IMessage message)
        {
            var type = message.GetType();

            messageTypeToMessages[type].Push(message);
        }

        internal void AddMessage<TMessage>(TMessage message) where TMessage : struct, IMessage
        {
            messageTypeToMessages[typeof(TMessage)].Push(message);
        }

        internal void ClearMessages()
        {
            foreach (var entry in messageTypeToMessages)
            {
                entry.Value.Clear();
            }
        }

        internal IEnumerable<TMessage> GetMessagesByType<TMessage>() where TMessage : struct, IMessage
        {
            return messageTypeToMessages.ContainsKey(typeof(TMessage)) ?
                messageTypeToMessages[typeof(TMessage)].Cast<TMessage>() :
                Enumerable.Empty<TMessage>();
        }
    }
}
